@@ -1,6 +1,6 @@
 package Treex::Block::Util::Eval;
-BEGIN {
-  $Treex::Block::Util::Eval::VERSION = '0.06903_1';
+{
+  $Treex::Block::Util::Eval::VERSION = '0.08302_1';
 }
 use Moose;
 use Treex::Core::Common;
@@ -50,15 +50,19 @@ sub process_document {
     }
 
     if ( $self->_args->{_bundle} ) {
+        my $bundleNo = 1;
         foreach my $bundle ( $document->get_bundles() ) {
-            $self->process_bundle($bundle);
+            if ( !$self->grep_bundle || $self->grep_bundle == $bundleNo ) {
+                $self->process_bundle($bundle, $bundleNo);
+            }
+            $bundleNo++;
         }
     }
     return;
 }
 
 sub process_bundle {
-    my ( $self, $bundle ) = @_;
+    my ( $self, $bundle, $bundleNo ) = @_;
 
     # Extract variables $document ($doc), so they can be used in eval code
     my $document = $bundle->get_document();
@@ -132,7 +136,7 @@ Treex::Block::Util::Eval - Special block for evaluating code given by parameters
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 SYNOPSIS
 
@@ -152,4 +156,4 @@ version 0.06903_1
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
