@@ -1,14 +1,15 @@
 package Treex::Block::Write::Sentences;
-BEGIN {
-  $Treex::Block::Write::Sentences::VERSION = '0.06903_1';
+{
+  $Treex::Block::Write::Sentences::VERSION = '0.08302_1';
 }
 use Moose;
 use Treex::Core::Common;
-extends 'Treex::Core::Block';
-with 'Treex::Block::Write::Redirectable';
+extends 'Treex::Block::Write::BaseTextWriter';
 
 has '+language' => ( required => 1 );
 
+has '+extension' => ( default => '.txt' );
+
 has join_resegmented => (
     is            => 'ro',
     isa           => 'Bool',
@@ -39,7 +40,7 @@ Treex::Block::Write::Sentences
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 DESCRIPTION
 
@@ -84,4 +85,4 @@ Martin Popel
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
