@@ -1,16 +1,20 @@
 package Treex::Block::Write::Text;
-BEGIN {
-  $Treex::Block::Write::Text::VERSION = '0.06903_1';
+{
+  $Treex::Block::Write::Text::VERSION = '0.08302_1';
 }
 use Moose;
 use Treex::Core::Common;
-extends 'Treex::Core::Block';
+extends 'Treex::Block::Write::BaseTextWriter';
 
-#TODO implement "to"
-has to => ( isa => 'Str', is => 'ro', default => '-' );
+has '+language' => ( required => 1 );
+
+has '+extension' => ( default => '.txt' );
 
 sub process_document {
     my ( $self, $doc ) = @_;
+
+    $self->_prepare_file_handle($doc);    # open the output file handle
+
     my $doczone = $doc->get_zone( $self->language, $self->selector );
     print $doczone->text;
     return;
@@ -26,7 +30,7 @@ Treex::Block::Write::Text
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 DESCRIPTION
 
@@ -65,4 +69,4 @@ Martin Popel
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
