@@ -1,6 +1,6 @@
 package Treex::Core::Block;
-BEGIN {
-  $Treex::Core::Block::VERSION = '0.06903_1';
+{
+  $Treex::Core::Block::VERSION = '0.08302_1';
 }
 use Moose;
 use Treex::Core::Common;
@@ -15,6 +15,14 @@ has scenario => (
     weak_ref => 1,
 );
 
+has grep_bundle => (
+    is            => 'ro',
+    isa           => 'Int',                                            # or regex in future?
+    default       => 0,
+    documentation => 'apply process_bundle only on the n-th bundle,'
+        . ' 0 (default) means apply to all bundles. Useful for debugging.',
+);
+
 # If the block name contains language (e.g. W2A::EN::Tokenize contains "en")
 # or target-language (e.g. T2T::CS2EN::FixNegation contains "en"),
 # it is returned as a default value of the attribute $self->language
@@ -54,10 +62,7 @@ sub BUILD {
 sub require_files_from_share {
     my ( $self, @rel_paths ) = @_;
     my $my_name = 'the block ' . $self->get_block_name();
-    foreach my $rel_path (@rel_paths) {
-        Treex::Core::Resource::require_file_from_share( $rel_path, $my_name );
-    }
-    return;
+    return map { Treex::Core::Resource::require_file_from_share( $_, $my_name ) } @rel_paths;
 }
 
 sub get_required_share_files {
@@ -79,16 +84,19 @@ sub process_document {
         log_fatal "There are no bundles in the document and block " . $self->get_block_name() .
             " doesn't override the method process_document";
     }
-    
+
     my $bundleNo = 1;
     foreach my $bundle ( $document->get_bundles() ) {
-        $self->process_bundle($bundle, $bundleNo++);
+        if ( !$self->grep_bundle || $self->grep_bundle == $bundleNo ) {
+            $self->process_bundle( $bundle, $bundleNo );
+        }
+        $bundleNo++;
     }
     return 1;
 }
 
 sub process_bundle {
-    my ($self, $bundle, $bundleNo) = @_;
+    my ( $self, $bundle, $bundleNo ) = @_;
 
     log_fatal "Parameter language was not set and block " . $self->get_block_name()
         . " doesn't override the method process_bundle" if !$self->language;
@@ -102,7 +110,7 @@ sub process_bundle {
             . " doesn't override the method process_bundle"
         )
         if !$zone;
-    return $self->process_zone($zone, $bundleNo);
+    return $self->process_zone( $zone, $bundleNo );
 }
 
 sub _try_process_layer {
@@ -120,7 +128,9 @@ sub _try_process_layer {
     }
 
     if ( my $m = $meta->find_method_by_name("process_${layer}node") ) {
-        foreach my $node ( $tree->get_descendants() ) {
+        ## process_ptree should be executed also on the root node (usually the S phrase)
+        my @opts = $layer eq 'p' ? ({add_self => 1}) : ();
+        foreach my $node ( $tree->get_descendants(@opts) ) {
             ##$self->process_anode($node);
             $m->execute( $self, $node, $bundleNo );
         }
@@ -131,7 +141,7 @@ sub _try_process_layer {
 }
 
 sub process_zone {
-    my ($self, $zone, $bundleNo) = @_;
+    my ( $self, $zone, $bundleNo ) = @_;
 
     my $overriden;
 
@@ -146,6 +156,13 @@ sub process_zone {
     return;
 }
 
+sub process_end {
+    my ($self) = @_;
+
+    # default implementation is empty, but can be overriden
+    return;
+}
+
 sub get_block_name {
     my $self = shift;
     return ref($self);
@@ -165,7 +182,7 @@ Treex::Core::Block - the basic data-processing unit in the Treex framework
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 SYNOPSIS
 
@@ -230,9 +247,33 @@ Applies the block instance on the given bundle zone
 C<process_document> and C<process_bundle>, C<process_zone> requires block 
 attribute C<language> (and possibly also C<selector>) to be specified.
 
+=item $block->process_I<X>tree($tree);
+
+Here I<X> stands for a,t,n or p.
+This method is executed on the root node of a tree on a given layer (a,t,n,p).
+
+=item $block->process_I<X>node($node);
+
+Here I<X> stands for a,t,n or p.
+This method is executed on the every node of a tree on a given layer (a,t,n,p).
+Note that for layers a, t, and n, this method is not executed on the root node
+(because the root node is just a "technical" root without the attributes of regular nodes).
+However, C<process_pnode> is executed also on the root node
+(because its a regular non-terminal node with a phrase attribute, usually C<S>).
 
 =back
 
+=head2 $block->process_end();
+
+This method is called after all documents are processed.
+The default implementation is empty, but derived classes can override it
+to e.g. print some final summaries, statistics etc.
+Overriding this method is preferable to both
+standard Perl END blocks (where you cannot access C<$self> and instance attributes),
+and DEMOLISH (which is not called in some cases, e.g. C<treex --watch>).
+
+
+
 =head1 BLOCK PARAMETRIZATION
 
 =over 4
@@ -263,7 +304,7 @@ It returns the name of the block module.
 
 If a block requires some files to be present in the shared part of Treex, 
 their list (with relative paths starting in 
-L<Treex::Core::Config::share_dir|Treex::Core::Config/share_dir>) can be 
+L<Treex::Core::Config->share_dir|Treex::Core::Config/share_dir>) can be 
 specified by redefining by this method. By default, an empty list is returned. 
 Presence of the files is automatically checked in the block constructor. If 
 some of the required file is missing, the constructor tries to download it 
@@ -303,4 +344,4 @@ Martin Popel <popel@ufal.mff.cuni.cz>
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
