@@ -1,6 +1,6 @@
 package Treex::Core::BundleZone;
-BEGIN {
-  $Treex::Core::BundleZone::VERSION = '0.06903_1';
+{
+  $Treex::Core::BundleZone::VERSION = '0.08302_1';
 }
 
 use Moose;
@@ -10,6 +10,7 @@ use MooseX::NonMoose;
 use Treex::Core::Node::A;
 use Treex::Core::Node::T;
 use Treex::Core::Node::N;
+use Treex::Core::Node::P;
 
 extends 'Treex::Core::Zone';
 
@@ -35,39 +36,56 @@ sub get_document {
 }
 
 sub create_atree {
-    my $self = shift;
-    return $self->create_tree('a');
+    my ($self,$params_rf) = @_;
+    return $self->create_tree('a',$params_rf);
 }
 
 sub create_ttree {
-    my $self = shift;
-    return $self->create_tree('t');
+    my ($self,$params_rf) = @_;
+    return $self->create_tree('t',$params_rf);
 }
 
 sub create_ntree {
-    my $self = shift;
-    return $self->create_tree('n');
+    my ($self,$params_rf) = @_;
+    return $self->create_tree('n',$params_rf);
 }
 
 sub create_ptree {
-    my $self = shift;
-    return $self->create_tree('p');
+    my ($self,$params_rf) = @_;
+    return $self->create_tree('p',$params_rf);
 }
 
 sub create_tree {
     my $self = shift;
-    my ($layer) = pos_validated_list(
+    my ($layer,$params_rf) = pos_validated_list(
         \@_,
         { isa => 'Treex::Type::Layer' },
+        { isa => 'Ref' },
     );
-    log_fatal("Zone already contains tree at $layer layer") if $self->has_tree($layer);
-    my $class = "Treex::Core::Node::" . uc($layer);
-    my $tree_root = eval { $class->new( { _called_from_core_ => 1 } ) } or log_fatal $!;    #layer subclasses not available yet
+    $layer = lc $layer;
+
+    if ($self->has_tree($layer)) {
+        if (defined $params_rf and $params_rf->{overwrite}) {
+            $self->remove_tree($layer);
+        }
+        else {
+            log_fatal("Zone already contains tree at $layer layer");
+        }
+    }
+
+    #my $class = "Treex::Core::Node::" . uc($layer);
+    #my $tree_root = eval { $class->new( { _called_from_core_ => 1 } ) } or log_fatal $!;    #layer subclasses not available yet
+    my $opts = { _called_from_core_ => 1 };
+    my $tree_root = $layer eq 'a' ? Treex::Core::Node::A->new($opts)
+                  : $layer eq 't' ? Treex::Core::Node::T->new($opts)
+                  : $layer eq 'n' ? Treex::Core::Node::N->new($opts)
+                  : $layer eq 'p' ? Treex::Core::Node::P->new($opts)
+                  : log_fatal "Cannot create tree for unknown layer $layer";
 
     my $bundle = $self->get_bundle;
     $tree_root->_set_zone($self);
 
-    my $new_tree_name = lc($layer) . "_tree";
+    my $new_tree_name = $layer . "_tree";
     $self->{trees}->{$new_tree_name} = $tree_root;
 
     my $new_id = "$new_tree_name-" . $self->get_label . "-" . $bundle->get_id . "-root";
@@ -200,10 +218,19 @@ sub copy {
     # Get the bundle the zone is in.
     my $bundle = $self->get_bundle();
     my $zone1 = $bundle->get_or_create_zone( $self->language(), $selector1 );
-    ### TO DO: copy other trees, too (currently only copies the a-tree)
-    my $aroot0 = $self->get_atree();
-    my $aroot1 = $zone1->create_atree();
-    $aroot0->copy_atree($aroot1);
+    ### TO DO: copy other trees, too (currently only copies a-tree and p-tree)
+    if($self->has_atree())
+    {
+        my $aroot0 = $self->get_atree();
+        my $aroot1 = $zone1->create_atree();
+        $aroot0->copy_atree($aroot1);
+    }
+    if($self->has_ptree())
+    {
+        my $proot0 = $self->get_ptree();
+        my $proot1 = $zone1->create_ptree();
+        $proot0->copy_ptree($proot1);
+    }
     return $zone1;
 }
 
@@ -221,7 +248,7 @@ Treex::Core::BundleZone - contains a sentence and its linguistic representations
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 SYNOPSIS
 
@@ -377,4 +404,4 @@ Martin Popel <popel@ufal.mff.cuni.cz>
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
