@@ -1,10 +1,10 @@
 package Treex::Core::Common;
-BEGIN {
-  $Treex::Core::Common::VERSION = '0.06903_1';
+{
+  $Treex::Core::Common::VERSION = '0.08302_1';
 }
 use strict;
 use warnings;
-use 5.008;
+use 5.010;
 
 use utf8;
 use Moose::Exporter;
@@ -39,7 +39,7 @@ sub pos_validated_list {
     return @{$args_ref};
 }
 
-# Choose which variant to use according to Treex::Core::Config::$params_validate
+# Choose which variant to use according to $Treex::Core::Config::params_validate
 if ( $Treex::Core::Config::params_validate == 2 ) {    ## no critic (ProhibitPackageVars)
     require MooseX::Params::Validate;
     $validation_sub = \&MooseX::Params::Validate::pos_validated_list;
@@ -74,6 +74,7 @@ my ( $import, $unimport, $init_meta ) =
     );
 
 sub import {
+    feature->import(':5.10');
     utf8::import();
     goto &$import;
 }
@@ -90,7 +91,7 @@ Treex::Core::Common - shorten the "C<use>" part of your Perl codes
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 SYNOPSIS
 
@@ -144,4 +145,4 @@ Martin Popel <popel@ufal.mff.cuni.cz>
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
