@@ -1,6 +1,6 @@
 package Treex::Core::Document;
-BEGIN {
-  $Treex::Core::Document::VERSION = '0.06903_1';
+{
+  $Treex::Core::Document::VERSION = '0.08302_1';
 }
 
 use Moose;
@@ -11,17 +11,26 @@ use Treex::Core::Bundle;
 
 use Treex::PML;
 Treex::PML::UseBackends('PMLBackend');
-Treex::PML::AddResourcePath( Treex::Core::Config::pml_schema_dir() );
+Treex::PML::AddResourcePath( Treex::Core::Config->pml_schema_dir() );
 
 with 'Treex::Core::WildAttr';
 
-use Scalar::Util qw( weaken );
+use Scalar::Util qw( weaken reftype );
+
+use PerlIO::via::gzip;
+use Storable;
 
 has loaded_from => ( is => 'rw', isa => 'Str', default => '' );
 has path        => ( is => 'rw', isa => 'Str' );
 has file_stem   => ( is => 'rw', isa => 'Str', default => 'noname' );
 has file_number => ( is => 'rw', isa => 'Str', builder => 'build_file_number' );
-has compress    => ( is => 'rw', isa => 'Bool', default => undef, documentation => 'compression to .gz' );
+has compress => ( is => 'rw', isa => 'Bool', default => undef, documentation => 'compression to .gz' );
+has storable => (
+    is            => 'rw',
+    isa           => 'Bool',
+    default       => undef,
+    documentation => 'using Storable with gz compression instead of Treex::PML'
+);
 
 has _pmldoc => (
     isa      => 'Treex::PML::Document',
@@ -44,7 +53,7 @@ has _pmldoc => (
             hint changeHint pattern_count pattern patterns
             changePatterns tail changeTail
 
-            trees changeTrees treeList tree lastTreeNo notSaved
+            trees changeTrees treeList tree delete_tree lastTreeNo notSaved
             currentTreeNo currentNode nodes value_line value_line_list
             determine_node_type )
     },
@@ -56,6 +65,11 @@ has _index => (
     default => sub { return {} },
 );
 
+has _backref => (
+    is => 'rw',
+    default => sub { return {} },
+);
+
 has _latest_node_number => (    # for generating document-unique IDs
     is      => 'rw',
     default => 0,
@@ -108,14 +122,22 @@ sub BUILD {
 
         # loading Treex::Core::Document from a file
         elsif ( $params_rf->{filename} ) {
-            # If the file contains invalid PML (e.g. unknown afun value)
-            # Treex::PML fails with die.
-            # TODO: we should rather catch the die message and report it via log_fatal
-            $pmldoc = eval {
-                $factory->createDocumentFromFile( $params_rf->{filename} );
-            };
-            log_fatal "Error while loading " . $params_rf->{filename}
-             if !defined $pmldoc;
+
+            if ( $params_rf->{filename} =~ /.streex$/ ) {
+                log_fatal 'Storable (.streex) docs must be retrieved by Treex::Core::Document->retrieve_storable($filename)';
+            }
+
+            else {
+
+                # If the file contains invalid PML (e.g. unknown afun value)
+                # Treex::PML fails with die.
+                # TODO: we should rather catch the die message and report it via log_fatal
+                $pmldoc = eval {
+                    $factory->createDocumentFromFile( $params_rf->{filename} );
+                };
+                log_fatal "Error while loading " . $params_rf->{filename}
+                    if !defined $pmldoc;
+            }
         }
     }
 
@@ -176,19 +198,6 @@ sub _rebless_and_index {
                     }
                     $tree->_set_zone($zone);
                 }
-
-                # TODO: Backward links from a-nodes to n-nodes
-                # should be created in n-nodes' constructors,
-                # which must be called after constructing a-nodes.
-                # TODO: Now, we don't call node constructors at all
-                # during loading, we just re-bless Treex::PML::Nodes.
-                if ( $zone->has_ntree ) {
-                    foreach my $nnode ( $zone->get_ntree()->get_descendants() ) {
-                        foreach my $anode ( $nnode->get_anodes() ) {
-                            $anode->_set_n_node($nnode);
-                        }
-                    }
-                }
             }
         }
     }
@@ -201,7 +210,7 @@ sub _pml_attribute_hash {
 }
 
 #my $_treex_schema_file = Treex::PML::ResolvePath( '.', 'treex_schema.xml', 1 );
-my $_treex_schema_file = Treex::Core::Config::pml_schema_dir . "/" . 'treex_schema.xml';
+my $_treex_schema_file = Treex::Core::Config->pml_schema_dir . "/" . 'treex_schema.xml';
 if ( not -f $_treex_schema_file ) {
     log_fatal "Can't find PML schema $_treex_schema_file";
 }
@@ -241,6 +250,11 @@ sub index_node_by_id {
     if ( defined $node ) {
         $index->{$id} = $node;
         weaken $index->{$id};
+
+        my $refs = $node->_get_referenced_ids;
+        foreach my $type ( keys %{$refs} ) {
+            $self->index_backref( $type, $id, $refs->{$type} );
+        }
     }
     else {
         delete $index->{$id};
@@ -248,6 +262,73 @@ sub index_node_by_id {
     return;
 }
 
+# Add references to the reversed references list
+sub index_backref {
+    my ( $self, $type, $source, $targets ) = @_;
+    my $backref = $self->_backref;
+
+    foreach my $target ( @{$targets} ) {
+        next if ( !defined($target) );
+        my $target_backrefs = $backref->{$target} // {};
+        $backref->{$target} = $target_backrefs;
+
+        $target_backrefs->{$type} = [] if ( !$target_backrefs->{$type} );
+        push @{ $target_backrefs->{$type} }, $source;
+    }
+    return;
+}
+
+# Remove references from the reversed references list
+sub remove_backref {
+    my ( $self, $type, $source, $targets ) = @_;
+    my $backref = $self->_backref;
+
+    foreach my $target ( @{$targets} ) {
+        next if ( !defined($target) );
+        my $target_backrefs = $backref->{$target};
+        next if ( !$target_backrefs );
+
+        $target_backrefs->{$type} = [ grep { $_ ne $source } @{ $target_backrefs->{$type} } ];
+    }
+    return;
+}
+
+# Return a hash of references ( type->[nodes] ) leading to the node with the given id
+sub get_references_to_id {
+    my ( $self, $id ) = @_;
+    my $backref = $self->_backref;
+
+    return if ( !$backref->{$id} );
+    return $backref->{$id};    # TODO clone ?
+}
+
+# Remove all references and backreferences leading to the $node (calls remove_reference() on the source nodes)
+sub _remove_references_to_node {
+    my ( $self, $node ) = @_;
+    my $id      = $node->id;
+    my $backref = $self->_backref;
+
+    # First, delete backreferences to the $node
+    my $refs = $node->_get_referenced_ids();
+    foreach my $type ( keys %{$refs} ) {
+        $self->remove_backref( $type, $id, $refs->{$type} );
+    }
+
+    # Second, delete references to the $node
+    return if ( !$backref->{$id} );
+    my $node_backref = $backref->{$id};
+
+    foreach my $type ( keys %{$node_backref} ) {
+        foreach my $source ( @{ $node_backref->{$type} } ) {
+            $self->get_node_by_id($source)->remove_reference( $type, $id );
+        }
+    }
+
+    # Third, delete backreferences from the $node
+    delete $backref->{$id};
+    return;
+}
+
 sub id_is_indexed {
     my $self = shift;
     my ($id) = pos_validated_list(
@@ -398,8 +479,25 @@ sub load {
 
 sub save {
     my $self = shift;
+    my ($filename) = @_;
 
-    # the following para should be some
+    if ( $filename =~ /\.streex$/ ) {
+        open( my $F, ">:via(gzip)", $filename ) or log_fatal $!;
+        print $F Storable::nfreeze( $self );
+        close $F;
+        # using  Storable::nstore_fd($self,*$F) emits 'Inappropriate ioctl for device'
+    }
+
+    else {
+        $self->_serialize_all_wild();
+        return $self->_pmldoc->save(@_);
+    }
+
+    return;
+}
+
+sub _serialize_all_wild {
+    my ($self) = @_;
     $self->serialize_wild;
     foreach my $bundle ( $self->get_bundles ) {
         $bundle->serialize_wild;
@@ -409,8 +507,40 @@ sub save {
             }
         }
     }
+    return;
+}
 
-    return $self->_pmldoc->save(@_);
+sub retrieve_storable {
+    my ( $class, $file ) = @_;    # $file stands for a file name, but it can be also file handle (needed by the TrEd backend for .streex)
+
+    my $FILEHANDLE;
+
+    my $stringified_doc;
+
+    if ( ref($file) and reftype($file) eq 'GLOB' ) {
+        $FILEHANDLE = $file;
+    }
+    else {
+        log_fatal "filename=$file, but Treex::Core::Document->retrieve(\$filename) can be used only for .streex files"
+            unless $file =~ /\.streex$/;
+        open $FILEHANDLE, "<:via(gzip)", $file or log_fatal($!);
+    }
+
+    my $serialized;
+    # reading it this way is silly, but both slurping the file or
+    #  using Storable::retrieve_fd lead to errors when used with via(gzip)
+    while (<$FILEHANDLE>) {
+        $serialized .= $_;
+    }
+    #    my $retrieved_doc = Storable::retrieve_fd(*$FILEHANDLE) or log_fatal($!);
+    my $retrieved_doc = Storable::thaw( $serialized ) or log_fatal $!;
+
+    if ( not ref($file) ) {
+        $retrieved_doc->set_loaded_from($file);
+        # $retrieved_doc->changeFilename($file); # why this doesn't affect the name displayed in TrEd?
+    }
+
+    return $retrieved_doc;
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -431,7 +561,7 @@ Treex::Core::Document - representation of a text and its linguistic analyses in
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 DESCRIPTION
 
@@ -562,6 +692,15 @@ Return C<true> if the given C<id> is already present in the indexing table.
 
 Return the array of all node identifiers indexed in the document.
 
+=item $document->get_references_to_id( $id );
+
+Return all references leading to the given node id in a hash (keys are reference types, e.g. 'alignment',
+'a/lex.rf' etc., values are arrays of nodes referencing this node).
+
+=item $document->remove_refences_to_id( $id );
+
+Remove all references to the given node id (calls remove_reference() on each referencing node).
+
 =back
 
 =head2 Serializing
@@ -574,7 +713,14 @@ Loads document from C<$filename> given C<%opts> using L<Treex::PML::Document::lo
 
 =item $document->save($filename)
 
-Saves document to C<$filename> using L<Treex::PML::Document::save()>
+Saves document to C<$filename> using L<Treex::PML::Document::save()>,
+or by the Storable module if the file's extension is .streex.gz.
+
+=item Treex::Core::Document->retrieve_storable($filename)
+
+Loading a document from the .streex (Storable) format.
+
+=back
 
 =head2 Other
 
@@ -593,8 +739,10 @@ Zdeněk Žabokrtský <zabokrtsky@ufal.mff.cuni.cz>
 
 Martin Popel <popel@ufal.mff.cuni.cz>
 
+Ondřej Dušek <odusek@ufal.mff.cuni.cz>
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
+Copyright © 2011-2012 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
