@@ -1,16 +1,38 @@
 package Treex::Core::Node;
-BEGIN {
-  $Treex::Core::Node::VERSION = '0.06903_1';
+{
+  $Treex::Core::Node::VERSION = '0.08302_1';
 }
 use Moose;
 use MooseX::NonMoose;
 use Treex::Core::Common;
 use Cwd;
+use Scalar::Util qw(refaddr);
 use Treex::PML;
 
 extends 'Treex::PML::Node';
 with 'Treex::Core::WildAttr';
 
+use overload
+    '""' => 'to_string',
+    '==' => 'equals',
+    '!=' => '_not_equals',
+    'eq' => 'equals',      # deprecated
+    'ne' => '_not_equals', # deprecated
+    'bool' => sub{1},
+
+    # We can A) let Magic Autogeneration to build "derived" overloadings,
+    # or B) we can disable this feature (via fallback=>0)
+    # and define only the needed overloadings
+    # (so all other overloadings will result in fatal errors).
+    # See perldoc overload.
+    # I decided for A, but uncommenting the following lines can catch some misuses.
+    #'!'  => sub{0},
+    #'.' => sub{$_[2] ? $_[1] . $_[0]->to_string : $_[0]->to_string . $_[1]},
+    #fallback => 0,
+;
+# TODO: '<' => 'precedes' (or better '<=>' => ...)
+# 'eq' => sub {log_warn 'You should use ==' && return $_[0]==$_[1]} # similarly for 'ne'
+
 Readonly my $_SWITCHES_REGEX => qr/^(ordered|add_self|(preceding|following|first|last)_only)$/x;
 my $CHECK_FOR_CYCLES = 1;
 
@@ -28,7 +50,11 @@ has id => (
 
 sub BUILD {
     my ( $self, $arg_ref ) = @_;
-    if ( not defined $arg_ref or not defined $arg_ref->{_called_from_core_} ) {
+
+    if (( not defined $arg_ref or not defined $arg_ref->{_called_from_core_} )
+        and not $Treex::Core::Config::running_in_tred
+        )
+    {
         log_fatal 'Because of node indexing, no nodes can be created outside of documents. '
             . 'You have to use $zone->create_tree(...) or $node->create_child() '
             . 'instead of Treex::Core::Node...->new().';
@@ -36,6 +62,24 @@ sub BUILD {
     return;
 }
 
+sub to_string {
+    my ($self) = @_;
+    return $self->id // 'node_without_id(addr=' . refaddr($self) . ')';
+}
+
+# Since we have overloaded stringification, we must overload == as well,
+# so you can use "if ($nodeA ==  $nodeB){...}".
+sub equals {
+    my ($self, $node) = @_;
+    #return ref($node) && $node->id eq $self->id;
+    return ref($node) && refaddr($node) == refaddr($self);
+}
+
+sub _not_equals {
+    my ($self, $node) = @_;
+    return !$self->equals($node);
+}
+
 sub _index_my_id {
     my $self = shift;
     pos_validated_list( \@_, { isa => 'Any', optional => 1 } );    #TODO
@@ -79,6 +123,25 @@ sub set_attr {
         $attr_value = Treex::PML::List->new( @{$attr_value} );
     }
 
+    if ($attr_name =~ /\.rf$/){
+        my $document = $self->get_document();
+
+        # Delete previous back references
+        my $old_value = $self->get_attr($attr_name);
+        if ($old_value) {
+            if ( ref $old_value eq 'Treex::PML::List' && @$old_value ) {
+                $document->remove_backref( $attr_name, $self->id, $old_value );
+            }
+            else {
+                $document->remove_backref( $attr_name, $self->id, [$old_value] );
+            }
+        }
+
+        # Set new back references
+        my $ids = ref($attr_value) eq 'Treex::PML::List' ? $attr_value : [$attr_value];
+        $document->index_backref( $attr_name, $self->id, $ids );
+    }
+
     #simple attributes can be accessed directly
     return $self->{$attr_name} = $attr_value if $attr_name =~ /^[\w\.]+$/;
     log_fatal "Attribute '$attr_name' contains strange symbols."
@@ -117,17 +180,21 @@ sub get_deref_attr {
 sub set_deref_attr {
     my ( $self, $attr_name, $attr_value ) = @_;
     log_fatal('Incorrect number of arguments') if @_ != 3;
+
+    # If $attr_value is an array of nodes
     if ( ref($attr_value) eq 'ARRAY' ) {
         my @list = map { $_->id } @{$attr_value};
         $attr_value = Treex::PML::List->new(@list);
     }
+
+    # If $attr_value is just one node
     else {
         $attr_value = $attr_value->id;
     }
 
-    # attr setting always through TectoMT set_attr, as it can be overidden (and it is in Node/N.pm)
-    #return $fsnode{ ident $self}->set_attr( $attr_name, $attr_value );
-    return $self->set_attr( $attr_name, $attr_value );
+    # Set the new reference(s)
+    $self->set_attr( $attr_name, $attr_value );
+    return;
 }
 
 sub get_bundle {
@@ -164,8 +231,10 @@ sub remove {
     my $document = $self->get_document();
 
     # Remove the subtree from the document's indexing table
-    foreach my $node ( $self, $self->get_descendants ) {
+    my @to_remove = ( $self, $self->get_descendants );
+    foreach my $node ( @to_remove) {
         if ( defined $node->id ) {
+            $document->_remove_references_to_node( $node );
             $document->index_node_by_id( $node->id, undef );
         }
     }
@@ -180,7 +249,49 @@ sub remove {
 
     # By reblessing we make sure that
     # all methods called on removed nodes will result in fatal errors.
-    bless $self, 'Treex::Core::Node::Deleted';
+    foreach my $node (@to_remove){
+        bless $node, 'Treex::Core::Node::Deleted';
+    }
+    return;
+}
+
+# Return all nodes that have a reference of the given type (e.g. 'alignment', 'a/lex.rf') to this node
+sub get_referencing_nodes {
+
+    my ( $self, $type ) = @_;
+    my $doc  = $self->get_document;
+    my $refs = $doc->get_references_to_id( $self->id );
+
+    return if ( !$refs || !$refs->{$type} );
+    return map { $doc->get_node_by_id($_) } @{ $refs->{$type} };
+}
+
+# Remove a reference of the given type to the given node. This will not remove a reverse reference from document
+# index, since it is itself called from when removing reverse references; use the API methods for the individual
+# references if you want to keep reverse references up-to-date.
+sub remove_reference {
+    my ( $self, $type, $id ) = @_;
+
+    if ( $type eq 'alignment' ) {    # handle alignment links separately
+
+        my $links = $self->get_attr('alignment');
+
+        if ($links) {
+            my $document = $self->get_document;
+            $self->set_attr( 'alignment', [ grep { $_->{'counterpart.rf'} ne $id } @{$links} ] );
+        }
+    }
+    else {
+        my $attr = $self->get_attr($type);
+        log_fatal "undefined attr $type (id=$id)" if !defined $attr;
+
+        if ( $attr eq $id || scalar( @{$attr} ) <= 1 ) {                # single-value attributes
+            $self->set_attr( $type, undef );
+        }
+        else {
+            $attr->delete_value($id);                                   # TODO : will it be always a Treex::PML::List? Looks like it works.
+        }
+    }
     return;
 }
 
@@ -287,6 +398,12 @@ sub is_root {
     return !$self->parent;
 }
 
+sub is_leaf {
+    log_fatal 'Incorrect number of arguments' if @_ != 1;
+    my $self = shift;
+    return not $self->firstson;
+}
+
 sub get_parent {
     log_fatal 'Incorrect number of arguments' if @_ != 1;
     my $self = shift;
@@ -307,7 +424,7 @@ sub set_parent {
 
     # We cannot detach a node by setting an undefined parent. The if statement below will die.
     # Let's inform the user where the bad call is.
-    log_fatal('Cannot attach the node to an undefined parent') if ( !defined($parent) );
+    log_fatal( 'Cannot attach the node ' . $self->id . ' to an undefined parent' ) if ( !defined($parent) );
     if ( $self == $parent || $CHECK_FOR_CYCLES && $parent->is_descendant_of($self) ) {
         my $id   = $self->id;
         my $p_id = $parent->id;
@@ -483,9 +600,24 @@ sub get_aligned_nodes {
     return ( undef, undef );
 }
 
+sub get_aligned_nodes_of_type {
+    my ( $self, $type_regex ) = @_;
+    my @nodes;
+    my ( $n_rf, $t_rf ) = $self->get_aligned_nodes();
+    return if !$n_rf;
+    my $iterator = List::MoreUtils::each_arrayref( $n_rf, $t_rf );
+    while ( my ( $node, $type ) = $iterator->() ) {
+        if ( $type =~ /$type_regex/ ) {
+            push @nodes, $node;
+        }
+    }
+    return @nodes;
+}
+
 sub is_aligned_to {
     my ( $self, $node, $type ) = @_;
-    return grep { $_ eq $node } $self->get_aligned_nodes( $node, $type ) ? 1 : 0;
+    log_fatal 'Incorrect number of parameters' if @_ != 3;
+    return any { $_ eq $node } $self->get_aligned_nodes_of_type( $node, $type ) ? 1 : 0;
 }
 
 sub delete_aligned_node {
@@ -493,7 +625,11 @@ sub delete_aligned_node {
     my $links_rf = $self->get_attr('alignment');
     my @links    = ();
     if ($links_rf) {
-        @links = grep { $_->{'counterpart.rf'} ne $node->id || $_->{'type'} ne $type } @$links_rf;
+        @links = grep {
+            $_->{'counterpart.rf'} ne $node->id
+                || ( defined($type) && defined( $_->{'type'} ) && $_->{'type'} ne $type )
+            }
+            @$links_rf;
     }
     $self->set_attr( 'alignment', \@links );
     return;
@@ -508,6 +644,21 @@ sub add_aligned_node {
     return;
 }
 
+# remove invalid alignment links (leading to unindexed nodes)
+sub update_aligned_nodes {
+
+    my ($self)   = @_;
+    my $doc      = $self->get_document();
+    my $links_rf = $self->get_attr('alignment');
+    my @new_links;
+
+    foreach my $link ( @{$links_rf} ) {
+        push @new_links, $link if ( $doc->id_is_indexed( $link->{'counterpart.rf'} ) );
+    }
+    $self->set_attr( 'alignment', \@new_links );
+    return;
+}
+
 #************************************
 #---- OTHER ------
 
@@ -527,40 +678,6 @@ sub get_depth {
 sub _normalize_node_ordering {
 }
 
-# Empty DESTROY method is a hack to get rid of the "Deep recursion warning"
-# in Treex::PML::Node::DESTROY and MooseX::NonMoose::Meta::Role::Class::_check_superclass_destructor.
-# Without this hack, you get the warning after creating a node with 99 or more children.
-# Deep recursion on subroutine "Class::MOP::Method::execute" at .../5.12.2/MooseX/NonMoose/Meta/Role/Class.pm line 183.
-sub DESTROY {
-}
-
-#*************************************
-#---- DEPRECATED & QUESTIONABLE ------
-
-sub disconnect {
-    my $self = shift;
-    log_warn( '$node->disconnect is deprecated, use $node->remove', 1 );
-    return $self->remove();
-}
-
-sub get_ordering_value {
-    my $self = shift;
-    log_warn( '$node->get_ordering_value is deprecated, use $node->ord', 1 );
-    return $self->ord;
-}
-
-sub set_ordering_value {
-    my $self = shift;
-    log_warn( '$node->set_ordering_value($ord) is deprecated, it should be private $node->_set_ord($n)', 1 );
-    my ($val) = pos_validated_list(
-        \@_,
-        { isa => 'Num' },    #or isa => 'Int' ??, or Positive Int?
-    );
-    $self->_set_ord($val);
-    return;
-}
-
-# proposal
 sub get_address {
     log_fatal 'Incorrect number of arguments' if @_ != 1;
     my $self     = shift;
@@ -574,13 +691,16 @@ sub get_address {
     return "$file##$position.$id";
 }
 
-# deprecated
-sub get_fposition {
-    my $self = shift;
-    log_warn("Method get_fposition is deprecated use get_address() instead");
-    return $self->get_address();
+# Empty DESTROY method is a hack to get rid of the "Deep recursion warning"
+# in Treex::PML::Node::DESTROY and MooseX::NonMoose::Meta::Role::Class::_check_superclass_destructor.
+# Without this hack, you get the warning after creating a node with 99 or more children.
+# Deep recursion on subroutine "Class::MOP::Method::execute" at .../5.12.2/MooseX/NonMoose/Meta/Role/Class.pm line 183.
+sub DESTROY {
 }
 
+#*************************************
+#---- DEPRECATED & QUESTIONABLE ------
+
 sub generate_new_id {    #TODO move to Core::Document?
     log_fatal 'Incorrect number of arguments' if @_ != 1;
     my $self = shift;
@@ -657,11 +777,40 @@ sub get_attrs {
     return @attr_values;
 }
 
+# Return all attributes of the given node (sub)type that contain references
+sub _get_reference_attrs {
+    my ($self) = @_;
+    return ();
+}
+
+# Return IDs of all nodes to which there are reference links from this node (must be overridden in
+# the respective node types)
+sub _get_referenced_ids {
+    my ($self) = @_;
+    my $ret = {};
+
+    # handle alignment separately
+    my $links_rf = $self->get_attr('alignment');
+    $ret->{alignment} = [ map { $_->{'counterpart.rf'} } @{$links_rf} ] if ($links_rf);
+
+    # all other references
+    foreach my $ref_attr ( $self->_get_reference_attrs() ) {
+        my $val = $self->get_attr($ref_attr) or next;
+        if ( !ref $val ) {    # single-valued
+            $ret->{$ref_attr} = [$val];
+        }
+        else {
+            $ret->{$ref_attr} = $val;
+        }
+    }
+    return $ret;
+}
+
 # TODO: How to do this in an elegant way?
 # Unless we find a better way, we must disable two perlcritics
 package Treex::Core::Node::Removed;
-BEGIN {
-  $Treex::Core::Node::Removed::VERSION = '0.06903_1';
+{
+  $Treex::Core::Node::Removed::VERSION = '0.08302_1';
 }    ## no critic (ProhibitMultiplePackages)
 use Treex::Core::Log;
 
@@ -728,7 +877,7 @@ sub set_r_attr {
 # ---------------------
 
 
-=for Pod::Coverage BUILD disconnect get_ordering_value set_ordering_value get_fposition
+=for Pod::Coverage BUILD
 
 
 =encoding utf-8
@@ -739,7 +888,7 @@ Treex::Core::Node - smallest unit that holds information in Treex
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 DESCRIPTION
 
@@ -858,10 +1007,14 @@ The removed node cannot be further used.
 
 Returns the root of the node's tree.
 
-=item my $root_node = $node->is_root();
+=item $node->is_root();
 
 Returns C<true> if the node has no parent.
 
+=item $node->is_leaf();
+
+Returns C<true> if the node has no children.
+
 =item $node1->is_descendant_of($node2);
 
 Tests whether C<$node1> is among transitive descendants of C<$node2>;
@@ -959,7 +1112,7 @@ Actually, this is shortcut for C<$node-E<gt>get_siblings({following_only=E<gt>1,
 
 =head2 PML-related methods
 
-=over 4
+=over
 
 =item my $type = $node->get_pml_type_name;
 
@@ -972,16 +1125,32 @@ Actually, this is shortcut for C<$node-E<gt>get_siblings({following_only=E<gt>1,
 
 =item add_aligned_node
 
-=item get_aligned_nodes
+=item my ($nodes_rf, $types_rf) = $node->get_aligned_nodes();
 
 Returns an array containing two array references. The first array contains the nodes aligned to this node, the second array contains types of the links.
 
+=item my @nodes = $node->get_aligned_nodes_of_type($regex_constraint_on_type);
+
 =item delete_aligned_node
 
 =item is_aligned_to
 
+=item update_aligned_nodes()
+
+Removes all alignment links leading to nodes which have been deleted.
+
 =back
 
+=head2 References (alignment and other references depending on node subtype)
+
+=over
+
+=item my @refnodes = $node->get_referencing_nodes($ref_type);
+
+Returns an array of nodes referencing this node with the given reference type (e.g. 'alignment', 'a/lex.rf' etc.).
+
+=back 
+
 =head2 Other methods
 
 =over 4
@@ -993,7 +1162,6 @@ nodes). The new identifier is derived from the identifier of the root
 (C<< $node->root >>), by adding suffix C<x1> (or C<x2>, if C<...x1> has already
 been indexed, etc.) to the root's C<id>.
 
-
 =item my $levels = $node->get_depth();
 
 Return the depth of the node. The root has depth = 0, its children have depth = 1 etc.
@@ -1003,6 +1171,18 @@ Return the depth of the node. The root has depth = 0, its children have depth =
 Return the node address, i.e. file name and node's position within the file,
 similarly to TrEd's C<FPosition()> (but the value is only returned, not  printed).
 
+=item $node->equals($another_node)
+
+This is the internal implementation of overloaded C<==> operator,
+which checks whether C<$node == $another_node> (the object instance must be identical).
+
+=item my $string = $node->to_string()
+
+This is the internal implementation of overloaded stringification,
+so you can use e.g. C<print "There is a node $node.">.
+It returns the id (C<$node->id>), but the behavior may be overridden in subclasses.
+See L<overload> pragma for details about overloading operators in Perl.
+
 =back
 
 
@@ -1014,8 +1194,12 @@ Martin Popel <popel@ufal.mff.cuni.cz>
 
 David Mareček <marecek@ufal.mff.cuni.cz>
 
+Daniel Zeman <zeman@ufal.mff.cuni.cz>
+
+Ondřej Dušek <odusek@ufal.mff.cuni.cz>
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
+Copyright © 2011-2012 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
