@@ -1,6 +1,6 @@
 package Treex::Core::Node::P;
-BEGIN {
-  $Treex::Core::Node::P::VERSION = '0.06903_1';
+{
+  $Treex::Core::Node::P::VERSION = '0.08302_1';
 }
 use Moose;
 use Treex::Core::Common;
@@ -9,7 +9,7 @@ extends 'Treex::Core::Node';
 # dirty: merging terminal and nonterminal nodes' attributes
 
 # common:
-has [qw(is_head is_collins_head head_selection_rule index coindex)] => ( is => 'rw' );
+has [qw(is_head index coindex edgelabel)] => ( is => 'rw' );
 
 # terminal specific
 has [qw(form lemma tag)] => ( is => 'rw' );
@@ -31,13 +31,10 @@ sub get_pml_type_name {
     }
 }
 
-# Nodes on the p-layer have no ordering attribute.
-# (It is not needed, trees are projective,
-#  the order is implied by the ordering of siblings.)
-override 'get_ordering_value' => sub {
-    my ($self) = @_;
-    return;
-};
+sub is_terminal {
+  my $self = shift @_;
+  return $self->get_pml_type_name eq 'p-terminal.type' ? 1 : 0;
+}
 
 sub create_nonterminal_child {
     my $self    = shift @_;
@@ -163,6 +160,58 @@ sub stringify_as_mrg {
     return "($string)";
 }
 
+sub stringify_as_text {
+    my ($self) = @_;
+    my @children = $self->get_children();
+    return $self->form // '<?>' if !@children;
+    return join ' ', map { $_->stringify_as_text() } @children;
+}
+
+#------------------------------------------------------------------------------
+# Recursively copy children from myself to another node.
+# This function is specific to the P layer because it contains the list of
+# attributes. If we could figure out the list automatically, the function would
+# become general enough to reside directly in Node.pm.
+#------------------------------------------------------------------------------
+sub copy_ptree
+{
+    my $self      = shift;
+    my $target    = shift;
+
+    # TODO probably we should do deepcopy
+    my %copy_of_wild = %{$self->wild};
+    $target->set_wild(\%copy_of_wild);
+
+    my @children0 = $self->get_children();
+    foreach my $child0 (@children0)
+    {
+
+        # Create a copy of the child node.
+        my $child1 = $child0->is_leaf() ? $target->create_terminal_child() : $target->create_nonterminal_child();
+
+        # We should copy all attributes that the node has but it is not easy to figure out which these are.
+        # TODO: As a workaround, we list the attributes here directly.
+        foreach my $attribute (
+            'form', 'lemma', 'tag', # terminal
+            'phrase', 'functions', # nonterminal
+            'edgelabel', 'is_head', 'index', 'coindex' # common
+            )
+        {
+            my $value = $child0->get_attr($attribute);
+            $child1->set_attr( $attribute, $value );
+        }
+
+        # TODO probably we should do deepcopy
+        %copy_of_wild = %{$child0->wild};
+        $child1->set_wild(\%copy_of_wild);
+
+        # Call recursively on the subtrees of the children.
+        $child0->copy_ptree($child1);
+    }
+
+    return;
+}
+
 1;
 
 __END__
@@ -175,20 +224,61 @@ Treex::Core::Node::P
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 DESCRIPTION
 
 Representation of nodes of phrase structure (constituency) trees.
 
+=head1 METHODS
+
+=head2 $node->is_terminal()
+
+Is C<$node> a terminal node?
+Does its C<get_pml_type_name eq 'p-terminal.type'>?
+
+=head2 my $child_phrase = $node->create_nonterminal_child()
+
+Create a new non-terminal child node,
+i.e. a node representing a constituent (phrase). 
+
+=head2 my $child_terminal = $node->create_terminal_child()
+
+Create a new terminal child node,
+i.e. a node representing a token. 
+
+=head2 my $node->create_from_mrg($mrg_string)
+
+Fill C<$node>'s attributes and create its subtree
+from the serialized string in the PennTB C<mrg> format.
+E.g.: I<(NP (DT a) (JJ nonexecutive) (NN director))>.
+
+=head2 my $mrg_string = $node->stringify_as_mrg()
+
+Serialize the tree structure of C<$node> and its subtree
+as a string in the PennTB C<mrg> format.
+E.g.: I<(NP (DT a) (JJ nonexecutive) (NN director))>.
+
+=head2 my $tokenized_text = $node->stringify_as_text()
+
+Get the text representing C<$node>'s subtree.
+The text is tokenized, i.e. all tokens are separated by a space.
+
+=head2 $node->copy_ptree($target_node)
+
+Recursively copy children from C<$node> to C<$target_node>.
+This method is specific to the P layer because it contains the list of
+attributes. If we could figure out the list automatically, the method would
+become general enough to reside directly in Node.pm.
 
 =head1 AUTHOR
 
 Martin Popel <popel@ufal.mff.cuni.cz>
-Zdenek Zabokrtsky <zabokrtsky@ufal.mff.cuni.cz>
+Zdeněk Žabokrtský <zabokrtsky@ufal.mff.cuni.cz>
+Daniel Zeman <zeman@ufal.mff.cuni.cz>
 
 =head1 COPYRIGHT AND LICENSE
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
