@@ -1,6 +1,6 @@
 package Treex::Core::Resource;
-BEGIN {
-  $Treex::Core::Resource::VERSION = '0.06903_1';
+{
+  $Treex::Core::Resource::VERSION = '0.08302_1';
 }
 use strict;
 use warnings;
@@ -9,8 +9,9 @@ use 5.010;
 
 #use Moose;
 #use Treex::Core::Common;
-use LWP::Simple;
-use File::Path;
+use LWP::Simple;    #TODO rewrite using LWP:UserAgent to show progress
+use File::Path 2.08 qw(make_path);
+use File::Spec;
 use Treex::Core::Log;
 use Treex::Core::Config;
 
@@ -22,24 +23,28 @@ sub require_file_from_share {
     my ( $rel_path_to_file, $who_wants_it, $make_executable ) = @_;
     my $writable;    #will store first writable directory found
     SEARCH:
-    foreach my $resource_dir ( Treex::Core::Config::resource_path() ) {
-        my $file = "$resource_dir/$rel_path_to_file";
+    foreach my $resource_dir ( Treex::Core::Config->resource_path() ) {
+        my $file = File::Spec->catfile( $resource_dir, $rel_path_to_file );
         log_debug("Trying $file\n");
         if ( -e $file ) {
             log_debug("Found $file\n");
             return $file;
         }
-
-        if ( !defined $writable && -w $resource_dir ) {
-            $writable = $resource_dir;
-            log_debug("Found writable directory: $writable");
+        if ( !defined $writable ) {
+            if ( !-e $resource_dir ) {
+                make_path($resource_dir);
+            }
+            if ( -d $resource_dir && -w $resource_dir ) {
+                $writable = $resource_dir;
+                log_debug("Found writable directory: $writable");
+            }
         }
     }
-    $who_wants_it = " by $who_wants_it" // '';
+    $who_wants_it = defined $who_wants_it ? " by $who_wants_it" : '';
     log_info("Shared file '$rel_path_to_file' is missing$who_wants_it.");
     log_fatal("Cannot find writable directory for downloading from share") if !defined $writable;
 
-    my $url = Treex::Core::Config::share_url(). "/$rel_path_to_file";
+    my $url = Treex::Core::Config->share_url() . "/$rel_path_to_file";
     log_info("Trying to download $url");
 
     my $file = "$writable/$rel_path_to_file";
@@ -82,6 +87,8 @@ Treex::Core::Resource - Access to shared resources
 
 =head1 VERSION
 
+version 0.08302_1
+
 =head1 SYNOPSIS
 
 use Treex::Core::Resource qw(require_file_from_share);
@@ -100,7 +107,7 @@ If not found, download from server (L<http://ufallab.ms.mff.cuni.cz/>)
 =item require_file_from_share($rel_path_to_file, $who_wants_it, $make_executable)
 
 Try to locate file in local resource paths, if not found, try to download it and stores it to first writable path.
-Obtains paths from L<Treex::Core::Config::resource_path()|Treex::Core::Config/resource_path>
+Obtains paths from L<Treex::Core::Config->resource_path()|Treex::Core::Config/resource_path>
 Returns path to file.
 
 =back
