@@ -1,6 +1,6 @@
 package Treex::Core::Run;
-BEGIN {
-  $Treex::Core::Run::VERSION = '0.06903_1';
+{
+  $Treex::Core::Run::VERSION = '0.08302_1';
 }
 use 5.008;
 use Moose;
@@ -14,6 +14,7 @@ use File::Path;
 use File::Temp qw(tempdir);
 use File::Which;
 use List::MoreUtils qw(first_index);
+use IO::Interactive;
 use Exporter;
 use base 'Exporter';
 our @EXPORT_OK = q(treex);
@@ -72,13 +73,6 @@ has 'selector' => (
     documentation => q{shortcut for adding "Util::SetGlobal selector=xy" at the beginning of the scenario},
 );
 
-has 'filelist' => (
-    traits        => ['Getopt'],
-    cmd_aliases   => 'l',
-    is            => 'rw', isa => 'Str',
-    documentation => 'TODO load a list of treex files from a file',
-);
-
 # treex -h should not print "Unknown option: h" before the usage.
 #has 'help' => (
 #    traits        => ['Getopt'],
@@ -157,6 +151,14 @@ has 'local' => (
     documentation => 'Run jobs locally (might help with multi-core machines). Requires -p.',
 );
 
+has 'priority' => (
+    traits        => ['Getopt'],
+    is            => 'ro',
+    isa           => 'Int',
+    default       => -100,
+    documentation => 'Priority for qsub, an integer in the range -1023 to 0 (or 1024 for admins), default=-100. Requires -p.',
+);
+
 has 'watch' => (
     traits        => ['Getopt'],
     is            => 'ro',
@@ -215,10 +217,16 @@ sub _usage_format {
 
 #gets info about version of treex and perl
 sub get_version {
-    my $perl_v         = $^V;
-    my $perl_x         = $^X;
-    my $treex_v        = $Treex::Core::Run::VERSION || 'DEV';
-    my $treex_x        = which('treex');
+    my $perl_v  = $^V;
+    my $perl_x  = $^X;
+    my $treex_v = $Treex::Core::Run::VERSION || 'DEV';
+    my $treex_x = which('treex');
+
+    # File::Which::which sometimes fails to found treex.
+    if ( !defined $treex_x ) {
+        chomp( $treex_x = `which treex 2> /dev/null` );
+        $treex_x ||= '<treex not found in $PATH>';
+    }
     my $version_string = <<"VERSIONS";
 Treex version: $treex_v from $treex_x
 Perl version: $perl_v from $perl_x
@@ -235,9 +243,6 @@ sub BUILD {
     }
 
     my @file_sources;
-    if ( $self->filelist ) {
-        push @file_sources, "filelist option";
-    }
     if ( $self->filenames ) {
         push @file_sources, "files after --";
     }
@@ -263,7 +268,7 @@ sub _execute {
         # TODO: execute_locally does the same work as the following line in a more safe ways
         # (If someone wants to run treex -d My::Block my_scen.scen)
         my $scen_str = join ' ', @{ $self->extra_argv };
-        $self->set_scenario( Treex::Core::Scenario->new( scenario_string => $scen_str ) );
+        $self->set_scenario( Treex::Core::Scenario->new( scenario_string => $scen_str, runner => $self ) );
 
         # TODO: Do it properly - perhaps, add a Scenario option to not load all the blocks.
         # We cannot create the real scenario instance without loading all the blocks
@@ -326,30 +331,36 @@ sub _execute {
 }
 
 my %READER_FOR = (
-    treex      => 'Treex',
+    'treex'    => 'Treex',
     'treex.gz' => 'Treex',
-    txt        => 'Text',
+    'txt'      => 'Text',
+    'txt.gz'   => 'Text',
+    'streex'   => 'Treex',
+    'mrg'      => 'PennMrg',
+    'mrg.gz'   => 'PennMrg',
 
     # TODO:
     # conll  => 'Conll',
     # plsgz  => 'Plsgz',
-    # treex.gz
     # tmt
 );
 
 sub _get_reader_name_for {
-    my $self  = shift;
-    my @names = @_;
-    my $re    = qr{\.(treex|txt)(\.gz)?$};
+    my $self    = shift;
+    my @names   = @_;
+    my $base_re = join( '|', keys %READER_FOR );
+    my $re      = qr{\.($base_re)$};
     my @extensions;
     my $first;
+
     foreach my $name (@names) {
-        if ( $name =~ $re ) {
+        if ( $name =~ /$re/ ) {
             my $current = $1;
-            if (!defined $first) {
+            $current =~ s/\.gz$//;
+            if ( !defined $first ) {
                 $first = $current;
             }
-            log_fatal 'All files (' . join( ',', @names ) . ') must have the same extension' if ($current ne $first);
+            log_fatal 'All files (' . join( ',', @names ) . ') must have the same extension' if ( $current ne $first );
             push @extensions, $current;
         }
         else {
@@ -367,7 +378,7 @@ sub _execute_locally {
     # Parameters can contain whitespaces that should be preserved
     my @arguments;
     foreach my $arg ( @{ $self->extra_argv } ) {
-        if ( $arg =~ /(\S+)=(.+\s.+)$/ ) {
+        if ( $arg =~ /(\S+)=(.*\s.*)$/ ) {
             my ( $name, $value ) = ( $1, $2 );
             $value =~ s/'/\\'/g;
             push @arguments, qq($name='$value');
@@ -386,18 +397,6 @@ sub _execute_locally {
         log_fatal "No files matching mask $mask" if @files == 0;
         $self->set_filenames( \@files );
     }
-    elsif ( $self->filelist ) {
-        open my $FL, "<:encoding(utf8)", $self->filelist
-            or log_fatal "Cannot open file list " . $self->filelist;
-        my @files;
-        while (<$FL>) {
-            chomp;
-            push @files, $_;
-        }
-        close $FL or log_fatal "Cannot close file list " . $self->filelist;
-        log_fatal q(No files matching mask ') . $self->glob . q('\n) if @files == 0;
-        $self->set_filenames( \@files );
-    }
 
     # some command line options are just shortcuts for blocks; the blocks are added to the scenario now
     if ( $self->filenames ) {
@@ -407,8 +406,8 @@ sub _execute_locally {
     }
 
     if ( $self->save ) {
-        log_info "Block Write::Treex added to the end of the scenario.";
-        $scen_str .= ' Write::Treex';
+        log_info "Block Write::Treex clobber=1 added to the end of the scenario.";
+        $scen_str .= ' Write::Treex clobber=1';
     }
 
     if ( $self->lang ) {
@@ -424,7 +423,7 @@ sub _execute_locally {
         $self->scenario->restart();
     }
     else {
-        $self->set_scenario( Treex::Core::Scenario->new( from_string => $scen_str ) );
+        $self->set_scenario( Treex::Core::Scenario->new( from_string => $scen_str, runner => $self ) );
         $self->scenario->load_blocks;
     }
     my $scenario      = $self->scenario;
@@ -510,9 +509,10 @@ sub _create_job_scripts {
     # You cannot use interactive input from terminal to "treex -p".
     # (If you really need it, use perl -npe 1 | treex -p ...)
     my $input = '';
-    if ( !-t STDIN ) {
+    if ( !IO::Interactive::is_interactive(*STDIN) ) {
         my $stdin_file = "$workdir/input";
         $input = "cat $stdin_file | ";
+        ## no critic (ProhibitExplicitStdin)
         open my $TEMP, '>', $stdin_file or log_fatal("Cannot create file $stdin_file to store input: $!");
         while (<STDIN>) {
             print $TEMP $_;
@@ -524,18 +524,20 @@ sub _create_job_scripts {
         my $script_filename = "scripts/job$jobnumber.sh";
         open my $J, ">", "$workdir/$script_filename" or log_fatal $!;
         print $J "#!/bin/bash\n\n";
-        print $J "echo \$HOSTNAME > $current_dir/$workdir/output/job$jobnumber.started\n";
+        print $J 'echo -e "$HOSTNAME\n"`date +"%s"` > ' . ( $workdir =~ /^\// ? $workdir : "$current_dir/$workdir" )
+            . "/output/job$jobnumber.started\n";
+        print $J "export PATH=/opt/bin/:\$PATH > /dev/null 2>&1\n\n";
         print $J "cd $current_dir\n\n";
-        print $J "source " . Treex::Core::Config::lib_core_dir()
+        print $J "source " . Treex::Core::Config->lib_core_dir()
             . "/../../../../config/init_devel_environ.sh 2> /dev/null\n\n";    # temporary hack !!!
 
         my $opts_and_scen = join ' ', map { _quote_argument($_) } @{ $self->ARGV };
         if ( $self->filenames ) {
             $opts_and_scen .= ' -- ' . join ' ', map { _quote_argument($_) } @{ $self->filenames };
         }
-        print $J $input . "treex --jobindex=$jobnumber --outdir=$workdir/output $opts_and_scen"
+        print $J $input . "treex --jobindex=$jobnumber --workdir=$workdir --outdir=$workdir/output $opts_and_scen"
             . " 2>> $workdir/output/job$jobnumber.started\n\n";
-        print $J "touch $workdir/output/job$jobnumber.finished\n";
+        print $J "date +'%s' > $workdir/output/job$jobnumber.finished\n";
         close $J;
         chmod 0777, "$workdir/$script_filename";
     }
@@ -555,7 +557,11 @@ sub _run_job_scripts {
             system "$workdir/$script_filename &";
         }
         else {
-            open my $QSUB, "cd $workdir && qsub -cwd " . $self->qsub . " -e output/ -S /bin/bash $script_filename |" or log_fatal $!;    ## no critic (ProhibitTwoArgOpen)
+            my $qsub_opts = '-cwd -e output/ -S /bin/bash ' . $self->qsub;
+            if ( $self->priority ) {
+                $qsub_opts .= ' -p ' . $self->priority;
+            }
+            open my $QSUB, "cd $workdir && qsub $qsub_opts $script_filename |" or log_fatal $!;    ## no critic (ProhibitTwoArgOpen)
 
             my $firstline = <$QSUB>;
             close $QSUB;
@@ -719,6 +725,71 @@ sub _wait_for_jobs {
     return;
 }
 
+sub _print_execution_time {
+    my ($self)              = @_;
+
+    my $time_total = 0;
+
+    my %hosts = ();
+
+    # read job log files
+    for my $file_finished (glob $self->workdir . "/output/job???.finished") {
+
+        # derivate file name
+        my $file_started = $file_finished;
+        $file_started =~ s/finished/started/;
+
+        # retrieve start time
+        open(my $fh_started, "<", $file_started) or log_fatal $!;
+        my $hostname = <$fh_started>;
+        chomp $hostname;
+        my $time_start = <$fh_started>;
+        close($fh_started);
+
+        # retrieve finish time
+        open(my $fh_finished, "<", $file_finished) or log_fatal $!;
+        my $time_finish = <$fh_finished>;
+        close($fh_finished);
+
+        # increase total time
+        $time_total += ( $time_finish - $time_start );
+        $hosts{$hostname}{'time'} += ( $time_finish - $time_start );
+        $hosts{$hostname}{'c'}++;
+    }
+
+    # find the slowest and the fastest machine
+    my $min_time = $time_total;
+    my $min_host = "";
+    my $max_time = 0;
+    my $max_host = 0;
+
+    for my $host (keys %hosts) {
+        my $avg = $hosts{$host}{'time'} / $hosts{$host}{'c'};
+        if ( $avg < $min_time ) {
+            $min_time = $avg;
+            $min_host = $host;
+        }
+
+        if ( $avg > $max_time ) {
+            $max_time = $avg;
+            $max_host = $host;
+        }
+    }
+
+    # print out statistics
+    log_info "Total execution time: $time_total";
+    log_info "Execution time per job: " . sprintf("%0.3f", $time_total / $self->jobs);
+    log_info "Slowest machine: $max_host = $max_time";
+    log_info "Fastest machine: $min_host = $min_time";
+
+    return;
+}
+
+# To get utf8 encoding also when using qx (aka backticks):
+# my $command_output = qw($command);
+# we need to
+use open qw{ :std IO :encoding(UTF-8) };
+
 sub _check_job_errors {
     my ($self) = @_;
     my $workdir = $self->workdir;
@@ -730,8 +801,14 @@ sub _check_job_errors {
         log_info "********************** FATAL ERRORS FOUND IN JOB $fatal_job ******************\n";
         log_info "$fatal_lines\n";
         log_info "********************** END OF JOB $fatal_job FATAL ERRORS LOG ****************\n";
-        log_info "All remaining jobs will be interrupted now.";
-        $self->_delete_jobs_and_exit;
+        if ( $self->survive ) {
+            log_warn("fatal error ignored due to the --survive option, be careful");
+            return;
+        }
+        else {
+            log_info "All remaining jobs will be interrupted now.";
+            $self->_delete_jobs_and_exit;
+        }
     }
     return;
 }
@@ -777,7 +854,7 @@ sub _execute_on_cluster {
     }
 
     foreach my $subdir (qw(output scripts)) {
-        mkdir $self->workdir . "/$subdir" or log_fatal $!;
+        mkdir $self->workdir . "/$subdir" or log_fatal 'Could not create directory ' . $self->workdir . "/$subdir : " . $!;
     }
 
     # catching Ctrl-C interruption
@@ -795,6 +872,8 @@ sub _execute_on_cluster {
 
     $self->_wait_for_jobs();
 
+    $self->_print_execution_time();
+
     log_info "All jobs finished.";
 
     if ( $self->cleanup ) {
@@ -890,7 +969,7 @@ Treex::Core::Run + treex - applying Treex blocks and/or scenarios on data
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 SYNOPSIS
 
@@ -931,7 +1010,7 @@ create new runner and runs scenario given in parameters
 
 =head1 USAGE
 
- usage: treex [-?dEegjLlpqSsv] [long options...] scenario [-- treex_files]
+ usage: treex [-?dEegjLpqSsv] [long options...] scenario [-- treex_files]
  scenario is a sequence of blocks or *.scen files
  options:
  	-? --usage --help            Prints this usage information.
@@ -951,8 +1030,6 @@ create new runner and runs scenario given in parameters
  	-S --selector                shortcut for adding "Util::SetGlobal
  	                             selector=xy" at the beginning of the
  	                             scenario
- 	-l --filelist                TODO load a list of treex files from a
- 	                             file
  	-g --glob                    Input file mask whose expansion is to
  	                             Perl, e.g. --glob '*.treex'
  	-p --parallel                Parallelize the task on SGE cluster
@@ -970,6 +1047,9 @@ create new runner and runs scenario given in parameters
  	                             Requires -p.
  	--local                      Run jobs locally (might help with
  	                             multi-core machines). Requires -p.
+ 	--priority                   Priority for qsub, an integer in the
+ 	                             range -1023 to 0 (or 1024 for admins),
+ 	                             default=-100. Requires -p.
  	--watch                      re-run when the given file is changed
  	                             TODO better doc
  	--workdir                    working directory for temporary files in
@@ -994,4 +1074,4 @@ Martin Popel <popel@ufal.mff.cuni.cz>
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
