@@ -1,12 +1,13 @@
 package Treex::Core::Scenario;
-BEGIN {
-  $Treex::Core::Scenario::VERSION = '0.06903_1';
+{
+  $Treex::Core::Scenario::VERSION = '0.08302_1';
 }
 use Moose;
 use Treex::Core::Common;
 use File::Basename;
 use File::Slurp;
 use File::chdir;
+use Parse::RecDescent 1.967003;
 
 has from_file => (
     is            => 'ro',
@@ -81,6 +82,14 @@ has parser => (
     documentation => q{Parses treex scenarios}
 );
 
+has runner => (
+    is            => 'ro',
+    isa           => 'Treex::Core::Run',
+    writer        => '_set_runner',
+    weak_ref      => 1,
+    documentation => 'Treex::Core::Run instance in which the scenario is running',
+);
+
 sub _build_scenario_string {
     my $self = shift;
     if ( $self->_has_from_file ) {
@@ -141,7 +150,6 @@ sub _build_parser {
         1;
     } and return $parser;
     log_info("Cannot find precompiled scenario parser, trying to build it from grammar");
-    use Parse::RecDescent;
     my $dir  = $self->_my_dir();             #get module's directory
     my $file = "$dir/ScenarioParser.rdg";    #find grammar file
     log_fatal("Cannot find grammar file") if !-e $file;
@@ -187,8 +195,8 @@ sub construct_scenario_string {
     my $delim       = $multiline ? qq{\n} : q{ };
     my @block_strings;
     foreach my $block_item (@block_items) {
-        my $name       = $block_item->{block_name};
-        my @parameters = @{ $block_item->{block_parameters} };
+        my $name = $block_item->{block_name};
+        my @parameters = map { _add_quotes($_) } @{ $block_item->{block_parameters} };
         $name =~ s{^Treex::Block::}{} or $name = "::$name";    #strip leading Treex::Block:: or add leading ::
         my $params;
         if ( scalar @parameters ) {
@@ -202,6 +210,15 @@ sub construct_scenario_string {
     return join $delim, @block_strings;
 }
 
+sub _add_quotes {    # adding quotes only if param. value contains a space
+    my ($block_parameter) = @_;
+    my ( $name, $value ) = split /=/, $block_parameter, 2;
+    if ( $value =~ /\s/ ) {
+        return "$name=\"$value\"";
+    }
+    return $block_parameter;
+}
+
 sub load_blocks {
     my $self = shift;
     $self->loaded_blocks;    #just access lazy attribute
@@ -264,6 +281,12 @@ sub run {
             print STDERR "Document $document_number/$number_of_documents $doc_name: [success].\n";
         }
     }
+
+    log_info "Applying process_end";
+    foreach my $block ( @{ $self->loaded_blocks } ) {
+        $block->process_end();
+    }
+
     log_info "Processed $document_number document"
         . ( $document_number == 1 ? '' : 's' );
     return 1;
@@ -297,7 +320,7 @@ Treex::Core::Scenario - a larger Treex processing unit, composed of blocks
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 SYNOPSIS
 
@@ -319,15 +342,15 @@ name is passed.
 
 The string description of scenarios looks as follows.
 
-1) It contains a list of block names from which their 'C<Treex::Block::>' 
+1) It contains a list of block names from which their 'C<Treex::Block::>'
 prefixes were removed.
 
-2) The block names are separated by one or more whitespaces.
+2) The block names are separated by one or more white spaces.
 
-3) The block names are listed in the same order in which they should be 
+3) The block names are listed in the same order in which they should be
 applied on data.
 
-4) For each block, there can be one or more parameters specified, using the 
+4) For each block, there can be one or more parameters specified, using the
 C<attribute=value> form.
 
 5) Comments start with 'C<#>' and end with the nearest newline character.
@@ -371,9 +394,9 @@ The scenario description is loaded from the file.
 =item $scenario->run();
 
 Run the scenario.
-One of the blocks (usually the first one) must be the document reader (see 
-L<Treex::Core::DocumentReader>) that produces the 
-documents on which this scenatio is applied.
+One of the blocks (usually the first one) must be the document reader (see
+L<Treex::Core::DocumentReader>) that produces the
+documents on which this scenario is applied.
 
 =back
 
@@ -402,12 +425,12 @@ when running scenario blocks are loaded automatically
 
 =item init
 
-do all initialization so after this method scenario is ready to run 
+do all initialization so after this method scenario is ready to run
 currently just load blocks
 
 =item restart
 
-resets document readed, in future it will rebuild reloaded blocks
+resets the document reader, in future it will rebuild reloaded blocks
 
 =back
 
@@ -431,4 +454,4 @@ Tomáš Kraut <kraut@ufal.mff.cuni.cz>
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
