@@ -1,6 +1,6 @@
 package Treex::Core::TredView;
-BEGIN {
-  $Treex::Core::TredView::VERSION = '0.06903_1';
+{
+  $Treex::Core::TredView::VERSION = '0.08302_1';
 }
 
 # planned to be used from contrib.mac of tred's extensions
@@ -14,6 +14,9 @@ use Treex::Core::TredView::Vallex;
 use Treex::Core::Types;
 use List::Util qw(first);
 
+use Treex::Core::Config;
+$Treex::Core::Config::running_in_tred = 1;
+
 has 'grp'       => ( is => 'rw' );
 has 'pml_doc'   => ( is => 'rw' );
 has 'treex_doc' => ( is => 'rw' );
@@ -48,6 +51,22 @@ has fast_loading => (
     documentation => 'Do the precomputation lazily for each bundle',
 );
 
+has _displayed_nodes => (
+    is      => 'rw',
+    isa     => 'HashRef[Treex::Core::Node]',
+    default => sub { {} }
+);
+has _ptb_index_map => (
+    is      => 'rw',
+    isa     => 'HashRef[Treex::Core::Node::P]',
+    default => sub { {} }
+);
+has _ptb_coindex_map => (
+    is      => 'rw',
+    isa     => 'HashRef[Treex::Core::Node::P]',
+    default => sub { {} }
+);
+
 has 'clause_collapsing' => ( is => 'rw', isa => 'Bool', default => 0 );
 has 'show_alignment'    => ( is => 'rw', isa => 'Bool', default => 1 );
 
@@ -90,6 +109,8 @@ sub get_nodelist_hook {
                                              # results in Can't locate object method "get_all_zones" via package "Treex::PML::Node" at /ha/work/people/popel/tectomt/treex/lib/Treex/Core/TredView.pm line 22
 
     my $layout = $self->tree_layout->get_layout();
+    $self->{'_ptb_index_map'}   = {};
+    $self->{'_ptb_coindex_map'} = {};
     my %nodes;
 
     foreach my $tree ( map { $_->get_all_trees } $bundle->get_all_zones ) {
@@ -98,6 +119,10 @@ sub get_nodelist_hook {
         if ( $tree->get_layer eq 'p' ) {
             @nodes = $self->_spread_nodes($tree);
             shift @nodes;
+            foreach my $node (@nodes) {
+                $self->{'_ptb_index_map'}->{ $node->{'index'} }     = $node if defined $node->{'index'};
+                $self->{'_ptb_coindex_map'}->{ $node->{'coindex'} } = $node if defined $node->{'coindex'};
+            }
         }
         elsif ( $tree->does('Treex::Core::Node::Ordered') ) {
             @nodes = $tree->get_descendants( { add_self => 1, ordered => 1 } );
@@ -129,8 +154,8 @@ sub get_nodelist_hook {
     for ( my $col = 0; $col < scalar @$layout; $col++ ) {
         my @task = ();
         for ( my $row = 0; $row < scalar @{ $layout->[$col] }; $row++ ) {
-            if ( $layout->[$col][$row] ) {
-                my $label     = $layout->[$col][$row];
+            if ( $layout->[$col][$row] and $layout->[$col][$row]->{'visible'} ) {
+                my $label     = $layout->[$col][$row]->{'label'};
                 my %tree_info = ();
                 $tree_info{'total'} = $tree_info{'left'} = scalar @{ $nodes{$label} };
                 $tree_info{'label'} = $label;
@@ -172,6 +197,8 @@ sub get_nodelist_hook {
         }
     }
 
+    $self->{'_displayed_nodes'} = { map { $_->attr('id') => 1 } @nodes };
+
     unless ( $currentNode and ( first { $_ == $currentNode } @nodes ) ) {
         $currentNode = $nodes[0];
     }
@@ -179,12 +206,31 @@ sub get_nodelist_hook {
     return [ \@nodes, $currentNode ];
 }
 
+sub recompute_visualization {
+    my ( $self, $bundle ) = @_;
+    $self->precompute_tree_depths($bundle);
+    $self->precompute_tree_shifts($bundle);
+    $self->precompute_visualization($bundle);
+    return;
+}
+
 sub file_opened_hook {
     my ($self) = @_;
     my $pmldoc = $self->grp()->{FSFile};
 
     $self->pml_doc($pmldoc);
-    my $treex_doc = Treex::Core::Document->new( { pmldoc => $pmldoc } );
+
+    my $treex_doc;
+    if ( defined $pmldoc->[13]->{_treex_core_document} ) {    # if it comes from storable (i.e., already with moose)
+        $treex_doc = $pmldoc->[13]->{_treex_core_document};
+
+        # streex files do not have "wild_dump" filled, but we want to show this in ttred
+        $treex_doc->_serialize_all_wild();
+    }
+    else {
+        $treex_doc = Treex::Core::Document->new( { pmldoc => $pmldoc } );
+    }
+
     $self->treex_doc($treex_doc);
 
     # labels, styles and vallex must be created again for each file
@@ -232,6 +278,7 @@ sub value_line_doubleclick_hook {
 
     my $layout = $self->tree_layout->get_layout;
     my %ordering;
+    my %visible;
     my $i     = 0;
     my $row   = 0;
     my $found = 1;
@@ -239,18 +286,25 @@ sub value_line_doubleclick_hook {
     while ($found) {
         $found = 0;
         for ( my $col = 0; $col < scalar @$layout; $col++ ) {
-            if ( $layout->[$col][$row] ) {
-                $ordering{ $layout->[$col][$row] } = $i++;
+            if ( defined $layout->[$col][$row] ) {
+                if ( $layout->[$col][$row]->{'visible'} ) {
+                    $ordering{ $layout->[$col][$row]->{'label'} } = $i++;
+                    $visible{ $layout->[$col][$row]->{'label'} }  = 1;
+                }
                 $found = 1;
             }
         }
         $row++;
     }
-    my @trees = sort { $ordering{ $self->tree_layout->get_tree_label($a) } <=> $ordering{ $self->tree_layout->get_tree_label($b) } } $bundle->get_all_trees;
+    my @trees = sort {
+        $ordering{ $self->tree_layout->get_tree_label($a) } <=> $ordering{ $self->tree_layout->get_tree_label($b) }
+        } grep {
+        exists $visible{ $self->tree_layout->get_tree_label($_) }
+        } $bundle->get_all_trees;
 
     for my $tree (@trees) {
         for my $node ( $tree->get_descendants ) {
-            next if $node->get_layer eq 'p' and $node->get_pml_type_name =~ m/nonterminal/;
+            next if $node->get_layer eq 'p' and not $node->is_terminal;
             return $node if exists $tags{"$node"};
         }
     }
@@ -307,7 +361,8 @@ sub precompute_tree_shifts {
         my $max_depth = 0;
         @trees = ();
         for ( my $col = 0; $col < scalar @$layout; $col++ ) {
-            if ( my $label = $layout->[$col][$row] ) {
+            if ( defined $layout->[$col][$row] ) {
+                my $label = $layout->[$col][$row]->{'label'};
                 my $depth = $forest{$label}{_tree_depth};
                 push @trees, $label;
                 $max_depth = $depth if $depth > $max_depth;
@@ -339,6 +394,10 @@ sub precompute_visualization {
                 $root->{_precomputed_node_style} = $self->_styles->node_style($root);
                 $root->{_precomputed_hint}       = '';
 
+                if ( $root->get_zone->sentence ) {
+                    $root->{_precomputed_hint} = 'sentence: ' . $root->get_zone->sentence;
+                }
+
                 foreach my $node ( $root->get_descendants ) {
                     $node->{_precomputed_node_style} = $self->_styles->node_style($node);
                     $node->{_precomputed_hint}       = $self->node_hint( $node, $layer );
@@ -359,7 +418,7 @@ sub precompute_visualization {
 }
 
 sub get_clickable_sentence_for_a_zone {
-    my ( $self, $zone ) = @_;
+    my ( $self, $zone, $alignment ) = @_;
     return if !$zone->has_atree();
     my %refs = ();
 
@@ -367,8 +426,16 @@ sub get_clickable_sentence_for_a_zone {
         for my $tnode ( $zone->get_ttree->get_descendants ) {
             for my $aux ( TredMacro::ListV( $tnode->attr('a/aux.rf') ) ) {
                 push @{ $refs{$aux} }, $tnode;
+                if ( exists $alignment->{ $tnode->get_attr('id') } ) {
+                    push @{ $refs{$aux} }, @{ $alignment->{ $tnode->get_attr('id') } };
+                }
+            }
+            if ( $tnode->attr('a/lex.rf') ) {
+                push @{ $refs{ $tnode->attr('a/lex.rf') } }, $tnode;
+                if ( exists $alignment->{ $tnode->get_attr('id') } ) {
+                    push @{ $refs{ $tnode->attr('a/lex.rf') } }, @{ $alignment->{ $tnode->get_attr('id') } };
+                }
             }
-            push @{ $refs{ $tnode->attr('a/lex.rf') } }, $tnode if $tnode->attr('a/lex.rf');
         }
     }
 
@@ -376,8 +443,11 @@ sub get_clickable_sentence_for_a_zone {
     for my $anode (@anodes) {
         my $id = $anode->id;
         push @{ $refs{$id} }, $anode;
-        if ( $anode->attr('p/terminal.rf') ) {
-            my $pnode = $self->treex_doc->get_node_by_id( $anode->attr('p/terminal.rf') );
+        if ( exists $alignment->{$id} ) {
+            push @{ $refs{$id} }, @{ $alignment->{$id} };
+        }
+        if ( $anode->attr('p_terminal.rf') ) {
+            my $pnode = $self->treex_doc->get_node_by_id( $anode->attr('p_terminal.rf') );
             push @{ $refs{$id} }, $pnode;
             while ( $pnode->parent ) {
                 $pnode = $pnode->parent;
@@ -388,7 +458,7 @@ sub get_clickable_sentence_for_a_zone {
 
     my @out;
     for my $anode (@anodes) {
-        push @out, [ $anode->form, @{ $refs{ $anode->id } || [] }, 'anode:' . $anode->id ];
+        push @out, [ $anode->form, @{ $refs{ $anode->id } || [] } ];
         if ( $anode->clause_number ) {
             my $clr = $self->_styles->_colors->get_clause_color( $anode->clause_number );
             push @{ $out[-1] }, "-foreground => $clr";
@@ -405,10 +475,27 @@ sub get_clickable_sentence_for_a_zone {
 sub precompute_value_line {
     my ( $self, $bundle ) = @_;
 
+    my %alignment = ();
+    foreach my $zone ( $bundle->get_all_zones() ) {
+        foreach my $layer (@layers) {
+            if ( $zone->has_tree($layer) ) {
+                my $root = $zone->get_tree($layer);
+                foreach my $node ( $root, $root->get_descendants ) {
+                    if ( exists $node->{'alignment'} ) {
+                        foreach my $ref ( @{ $node->get_attr('alignment') } ) {
+                            push @{ $alignment{ $node->{id} } }, $self->treex_doc->get_node_by_id( $ref->{'counterpart.rf'} );
+                            push @{ $alignment{ $ref->{'counterpart.rf'} } }, $node;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
     my @out = ();
     foreach my $zone ( $bundle->get_all_zones() ) {
         push @out, ( [ '[' . $zone->get_label . ']', 'label' ], [ ' ', 'space' ] );
-        if ( my $sentence = $self->get_clickable_sentence_for_a_zone($zone) ) {
+        if ( my $sentence = $self->get_clickable_sentence_for_a_zone( $zone, \%alignment ) ) {
             push @out, @$sentence;
         }
         elsif ( defined $zone->sentence ) {
@@ -449,7 +536,7 @@ sub anode_hint {
 
     push @lines, "Parenthesis root" if $node->{is_parenthesis_root};
     if ( $node->language eq 'cs' ) {
-        push @lines, "Full lemma: " . $node->{lemma};
+        push @lines, "Full lemma: " . ( $node->{lemma} ? $node->{lemma} : '' );
         push @lines, "Full tag: " . ( $node->{tag} ? $node->{tag} : '' );
     }
 
@@ -470,7 +557,7 @@ sub tnode_hint {
 
     if ( ref $node->get_attr('gram') ) {
         foreach my $gram ( keys %{ $node->get_attr('gram') } ) {
-            push @lines, "gram/$gram : " . $node->get_attr( 'gram/' . $gram );
+            push @lines, "gram/$gram : " . ( $node->get_attr( 'gram/' . $gram ) || '' );
         }
     }
 
@@ -491,11 +578,9 @@ sub nnode_hint {
 
 sub pnode_hint {
     my ( $self, $node ) = @_;
-
     my @lines = ();
-    my $terminal = $node->get_pml_type_name eq 'p-terminal.type' ? 1 : 0;
 
-    if ($terminal) {
+    if ( $node->is_terminal ) {
         push @lines, map { "$_: " . ( defined $node->{$_} ? $node->{$_} : '' ) } qw(lemma tag form);
     }
     else {
@@ -527,11 +612,39 @@ sub node_style_hook {
         }
     }
 
+    # P-layer indexes and coindexes
+    if ( $node->get_layer eq 'p' ) {
+        my $coindex;
+        if ( $node->attr('form') and $node->attr('form') =~ m/-(\d+)$/ ) {
+            $coindex = $1;
+        }
+        elsif ( $node->attr('coindex') ) {
+            $coindex = $node->attr('coindex');
+        }
+
+        if ($coindex) {
+            my $target_node;
+            if ( exists $self->{'_ptb_index_map'}->{$coindex} ) {
+                $target_node = $self->{'_ptb_index_map'}->{$coindex};
+            }
+            elsif ( $node->is_terminal and exists $self->{'_ptb_coindex_map'}->{$coindex} ) {
+                $target_node = $self->{'_ptb_coindex_map'}->{$coindex};
+            }
+
+            if ($target_node) {
+                push @target_ids,  $target_node->{'id'};
+                push @arrow_types, 'coindex';
+            }
+        }
+    }
+
     # alignment
     if ( $self->show_alignment and my $links = $node->attr('alignment') ) {
         foreach my $link (@$links) {
-            push @target_ids,  $link->{'counterpart.rf'};
-            push @arrow_types, 'alignment';
+            if ( exists $self->_displayed_nodes->{ $link->{'counterpart.rf'} } ) {
+                push @target_ids,  $link->{'counterpart.rf'};
+                push @arrow_types, 'alignment';
+            }
         }
     }
 
@@ -627,11 +740,13 @@ sub toggle_clause_collapsing {
             }
         }
     }
+    return;
 }
 
 sub toggle_alignment {
     my $self = shift;
     $self->show_alignment( not $self->show_alignment );
+    return;
 }
 
 1;
@@ -646,7 +761,7 @@ Treex::Core::TredView - visualization of Treex files in TrEd
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 DESCRIPTION
 
@@ -664,7 +779,7 @@ thickness) should be used for them.
 
 The TrEd visualization is precomputed statically after a file is loaded,
 therefore the extension can be currently used only for browsing, not for
-editting the treex files.
+editing the treex files.
 
 =head1 METHODS
 
@@ -708,4 +823,4 @@ Martin Popel <popel@ufal.mff.cuni.cz>
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
