@@ -1,6 +1,6 @@
 package Treex::Core::TredView::Colors;
-BEGIN {
-  $Treex::Core::TredView::Colors::VERSION = '0.06903_1';
+{
+  $Treex::Core::TredView::Colors::VERSION = '0.08302_1';
 }
 
 use Moose;
@@ -17,7 +17,7 @@ sub _build_colors {
         'edge'      => '#555555',
         'coord'     => '#bbbbbb',
         'error'     => '#ff0000',
-        'coord_mod' => '#666666',
+        'coord_mod' => 'aquamarine4',
 
         'anode'            => '#ff6666',
         'anode_coord'      => '#ff6666',
@@ -25,6 +25,7 @@ sub _build_colors {
         'tnode'            => '#4488ff',
         'tnode_coord'      => '#ccddff',
         'terminal'         => '#ffff66',
+        'terminal_head'    => '#90ee90',
         'nonterminal_head' => '#90ee90',
         'nonterminal'      => '#ffffe0',
         'trace'            => '#aaaaaa',
@@ -34,6 +35,7 @@ sub _build_colors {
         'coref_text' => '#4c509f',
         'compl'      => '#629f52',
         'alignment'  => '#bebebe',
+        'coindex'    => '#ffa500', #orange
 
         'lex'         => '#006400',
         'aux'         => '#ff8c00',
@@ -45,6 +47,9 @@ sub _build_colors {
         'nodetype'    => '#00008b',
         'sempos'      => '#8b008b',
         'phrase'      => '#00008b',
+        'formeme'     => '#b000b0',
+        'tag'         => '#004048',
+        'tag_feat'    => '#7098A0',
 
         'clause0' => '#ff00ff',         #magenta
         'clause1' => '#ffa500',         #orange
