@@ -1,6 +1,6 @@
 package Treex::Core::TredView::Labels;
-BEGIN {
-  $Treex::Core::TredView::Labels::VERSION = '0.06903_1';
+{
+  $Treex::Core::TredView::Labels::VERSION = '0.08302_1';
 }
 
 use Moose;
@@ -206,7 +206,12 @@ sub _anode_labels {
     if ($par) {
         $line1 = $self->_colors->get( 'parenthesis', 1 );
     }
-    $line1 .= $node->form;
+    if ( defined($node->form) ){
+        $line1 .= $node->form;
+    }
+    elsif ( defined($node->lemma) ){
+        $line1 .= $self->_colors->get( 'error', 1 ) . $node->lemma;
+    }
 
     my $edge_label = $node->afun || $node->conll_deprel;
     my $color = $edge_label && $edge_label ne 'NR' ? $self->_colors->get( 'afun', 1 ) : $self->_colors->get( 'error', 1 );
@@ -222,17 +227,19 @@ sub _anode_labels {
         }
     }
 
-    my $line3_1 = $node->tag ? $node->tag : "";
-    my $line3_2 = $node->lemma;
+    my $line3_1 = $node->tag ? $node->tag : '';
+    my $line3_2 = $node->lemma ? $node->lemma : '';
 
     # DZ: This hack tries to distinguish Dan's CoNLL trees from Pepa's PEDT trees
     #     so that Czech tags don't get crippled in the former.
     # if ( $node->language eq 'cs' ) {
     if ( $node->language eq 'cs' && !$node->conll_cpos ) {
-        $line3_1 = substr( $line3_1, 0, 2 );
+        $line3_1 =  $self->_shorten_czech_tag( $line3_1 );
         $line3_2 =~ s/(.)(?:-[1-9][0-9]*)?(?:(?:`|_[:;,^]).*)?$/$1/;
     }
 
+    $line3_1 = $self->_colors->get( 'tag', 1 ) . $line3_1;
+
     return [
         [$line1],
         [$line2],
@@ -240,6 +247,28 @@ sub _anode_labels {
     ];
 }
 
+sub _shorten_czech_tag {
+    my ($self, $tag) = @_;
+     
+    # nouns, adjectives, pronouns, declinable numerals: gender, number, case
+    if ($tag =~ m/^(N|C[adhjklnrwyz\?]|P|A)/){ 
+        $tag = substr( $tag, 0, 2 ) . $self->_colors->get( 'tag_feat', 1 ) . substr( $tag, 2, 3 );
+    }
+    # prepositions: just case
+    elsif ($tag =~ m/^R/ ){
+        $tag = substr( $tag, 0, 2 ) . $self->_colors->get( 'tag_feat', 1 ) . substr( $tag, 4, 1 );
+    }
+    # verbs (except infinitives, conditionals): gender, number, person, tense, voice
+    elsif ($tag =~ m/^V[^cf]/ ){
+        $tag = substr( $tag, 0, 2)  . $self->_colors->get( 'tag_feat', 1 ) 
+            . substr( $tag, 2, 2 ) . substr( $tag, 7, 2 ) . substr( $tag, 11, 1 );
+    }
+    else {
+        $tag = substr( $tag, 0, 2 );
+    }
+    return $tag;
+}
+
 sub _tnode_labels {
     my ( $self, $node ) = @_;
 
@@ -267,6 +296,7 @@ sub _tnode_labels {
     $line2 .= $self->_colors->get( 'subfunctor', 1 ) . '.state'                  if $node->{is_state};
     $line2 .= $self->_colors->get( 'subfunctor', 1 ) . '.dsp_root'               if $node->{is_dsp_root};
     $line2 .= $self->_colors->get( 'member',     1 ) . '.member'                 if $node->{is_member};
+    $line2 .= $self->_colors->get( 'formeme',    1 ) . ' ' . $node->formeme      if $node->formeme;
 
     my @a_nodes = ();
     my $line3_1 = '';
@@ -281,7 +311,7 @@ sub _tnode_labels {
     }
     if (@a_nodes) {
         @a_nodes = sort { $a->{ord} <=> $b->{ord} } @a_nodes;
-        @a_nodes = map { ( $_->{type} eq 'lex' ? $self->_colors->get( 'lex', 1 ) : $self->_colors->get( 'aux', 1 ) ) . $_->{form} } @a_nodes;
+        @a_nodes = map { ( $_->{type} eq 'lex' ? $self->_colors->get( 'lex', 1 ) : $self->_colors->get( 'aux', 1 ) ) . ( $_->{form} // '' ) } @a_nodes;
         $line3_1 = join " ", @a_nodes;
     }
 
@@ -307,17 +337,20 @@ sub _nnode_labels {
 sub _pnode_labels {
     my ( $self, $node ) = @_;
 
-    my $terminal = $node->get_pml_type_name eq 'p-terminal.type' ? 1 : 0;
+#    my $terminal = $node->get_pml_type_name eq 'p-terminal.type' ? 1 : 0;
 
     my $line1 = '';
     my $line2 = '';
-    if ($terminal) {
+    my $edgelabel = $node->edgelabel() ? '/'.$node->edgelabel() : '';
+#    print "$node is_leaf ".$node->is_leaf."\n";
+    if ( $node->is_leaf ) {
         $line1 = $node->{form};
-        $line2 = $node->{tag};
+        $line2 = $node->{tag}.$edgelabel;
         $line2 = '-' if $line2 eq '-NONE-';
     }
     else {
-        $line1 = $self->_colors->get( 'phrase', 1 ) . $node->{phrase} . '#{black}' . join( '', map {"-$_"} TredMacro::ListV( $node->{functions} ) );
+        my $phrase = $node->{phrase} ? $node->{phrase} : '';
+        $line1 = $self->_colors->get( 'phrase', 1 ) . $phrase.$edgelabel . '#{black}' . join( '', map {"-$_"} TredMacro::ListV( $node->{functions} ) );
     }
 
     return [
@@ -337,7 +370,7 @@ Treex::Core::TredView::Labels - Labels of tree nodes in Tred
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 DESCRIPTION
 
@@ -392,4 +425,4 @@ Josef Toman <toman@ufal.mff.cuni.cz>
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
