@@ -1,6 +1,6 @@
 package Treex::Core::TredView::Vallex;
-BEGIN {
-  $Treex::Core::TredView::Vallex::VERSION = '0.06903_1';
+{
+  $Treex::Core::TredView::Vallex::VERSION = '0.08302_1';
 }
 
 use Moose;
@@ -24,6 +24,7 @@ sub _extension_missing {
     my $message = "This function requires additional extension that is missing.\n";
     $message .= "Please install extension '$name'.";
     TredMacro::ToplevelFrame->messageBox( -type => 'ok', -message => $message );
+    return;
 }
 
 sub _find_vallex {
@@ -140,15 +141,14 @@ sub _OpenValLexicon_En {
     my $vallex_file = $self->_find_vallex($node);
     return unless $vallex_file;
 
-    TrEd::EngValLex::GUI::OpenEditor(
-        {
-            -vallex_file => $vallex_file,
-            -lemma       => $node->attr('t_lemma'),
-            -sempos      => $node->attr('gram/sempos'),
-            -frameid     => $node->attr('val_frame.rf'),
-            -pos         => $a_node->attr('tag')
-        }
-    );
+    my $opts = {
+        -vallex_file => $vallex_file,
+        -lemma       => $node->attr('t_lemma'),
+        -sempos      => $node->attr('gram/sempos'),
+        -frameid     => $node->attr('val_frame.rf')
+    };
+    $opts->{-pos} = $a_node->attr('tag') if $a_node->attr('tag');
+    TrEd::EngValLex::GUI::OpenEditor($opts);
     TredMacro::ChangingFile(0);
     return;
 }
@@ -188,6 +188,7 @@ sub _OpenValFrameList_Cs {
         }
     );
     TredMacro::ChangingFile(0);
+    return;
 }
 
 sub _OpenValFrameList_En {
@@ -214,20 +215,20 @@ sub _OpenValFrameList_En {
     my $vallex_file = $self->_find_vallex($node);
     return unless $vallex_file;
 
-    TrEd::EngValLex::GUI::ChooseFrame(
-        {
-            -withdraw    => 1,
-            -vallex_file => $vallex_file,
-            -lemma       => $node->{t_lemma} || undef,
-            -sempos      => $node->attr('gram/sempos') || undef,
-            -lemma_attr  => 't_lemma',
-            -sempos_attr => 'gram/sempos',
-            -frameid     => $node->attr('val_frame.rf'),
-            -assignfunc  => sub { },
-            -pos         => $a_node->attr('tag')
-        }
-    );
+    my $opts = {
+        -withdraw    => 1,
+        -vallex_file => $vallex_file,
+        -lemma       => $node->{t_lemma} || undef,
+        -sempos      => $node->attr('gram/sempos') || undef,
+        -lemma_attr  => 't_lemma',
+        -sempos_attr => 'gram/sempos',
+        -frameid     => $node->attr('val_frame.rf'),
+        -assignfunc  => sub { }
+    };
+    $opts->{-pos} = $a_node->attr('tag') if $a_node->attr('tag');
+    TrEd::EngValLex::GUI::ChooseFrame($opts);
     TredMacro::ChangingFile(0);
+    return;
 }
 
 1;
@@ -236,15 +237,15 @@ __END__
 
 =head1 NAME
 
-Treex::Core::TredView::Vallex - Browsing vallency lexicons
+Treex::Core::TredView::Vallex - Browsing valency lexicons
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =head1 DESCRIPTION
 
-This packages provides browsers of vallency lexicons
+This package provides browsers of valency lexicons
 
 =head1 METHODS
 
@@ -286,4 +287,4 @@ Josef Toman <toman@ufal.mff.cuni.cz>
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
