@@ -1,6 +1,6 @@
 package Treex::Core::WildAttr;
-BEGIN {
-  $Treex::Core::WildAttr::VERSION = '0.06903_1';
+{
+  $Treex::Core::WildAttr::VERSION = '0.08302_1';
 }
 use Moose::Role;
 
@@ -62,7 +62,7 @@ sub serialize_wild {
 sub deserialize_wild {
     my ($self) = @_;
     if ( $self->_wild_dump ) {
-        $self->set_wild( eval "my " . $self->_wild_dump . '; return $VAR1' );
+        $self->set_wild( eval "my " . $self->_wild_dump . '; return $VAR1' ); ## no critic (ProhibitStringyEval)
     }
     else {
         $self->set_wild( {} );
@@ -78,11 +78,11 @@ __END__
 
 =head1 NAME
 
-Treex::Core::WildAttr - role for arbitrary atributes of Treex objects
+Treex::Core::WildAttr - role for arbitrary attributes of Treex objects
 
 =head1 VERSION
 
-version 0.06903_1
+version 0.08302_1
 
 =for test_synopsis my ($node, $value);
 =head1 SYNOPSIS
@@ -150,4 +150,4 @@ Zdeněk Žabokrtský <zabokrtsky@ufal.mff.cuni.cz>
 
 Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
